'use strict';

exports.__esModule = true;
exports.default = undefined;

var _class, _temp2;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _browserEssentials = require('browser-essentials');

var _optSetter = require('opt-setter');

var _routeMatcher = require('./routeMatcher');

var routeMatcher = _interopRequireWildcard(_routeMatcher);

var _Html5History = require('./Html5History');

var _Html5History2 = _interopRequireDefault(_Html5History);

var _RouterContext = require('./RouterContext');

var _RouterContext2 = _interopRequireDefault(_RouterContext);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Router = (_temp2 = _class = function (_React$Component) {
  _inherits(Router, _React$Component);

  function Router() {
    var _temp, _this, _ret;

    _classCallCheck(this, Router);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
      location: null,
      route: { path: null, params: {}, view: null },
      routes: new Map()
    }, _this._listeners = [], _temp), _possibleConstructorReturn(_this, _ret);
  }

  Router.prototype.componentWillMount = function componentWillMount() {
    var location = this.props.initialLocation || this.props.history.location;
    var routes = routeMatcher.findRoutes(this.props.children);
    var route = this.getRoute(routes, location);

    this.setState({ location: location, routes: routes, route: route });
  };

  Router.prototype.componentDidMount = function componentDidMount() {
    var gotoListener = this.handleGoto.bind(this);
    this.props.history.on('navigate', gotoListener);

    var clickListener = this.handleClick.bind(this);
    document.addEventListener('click', clickListener);

    this._listeners.push(gotoListener, clickListener);
  };

  Router.prototype.componentWillUnmount = function componentWillUnmount() {
    var _listeners = this._listeners,
        gotoListener = _listeners[0],
        clickListener = _listeners[1];


    this.props.history.dispose();
    document.removeEventListener('click', clickListener);
  };

  Router.prototype.componentDidUpdate = function componentDidUpdate() {
    this.props.history.emit('routerUpdated', this.props.history.location);
  };

  Router.prototype.handleGoto = function handleGoto(location) {
    var route = this.getRoute(this.state.routes, location);
    this.setState({ location: location, route: route });
  };

  Router.prototype.handleClick = function handleClick(e) {
    // Ignores middle, ctrl, cmd and shift clicks
    if (_browserEssentials.dom.isMetaClick(e)) {
      return;
    }

    // Only targets A tags
    var target = e.target.nodeName === 'A' ? e.target : _browserEssentials.dom.findParentNode(e.target, 'A', this.props.maxClickSearchDepth);

    if (target === void 0) return;

    // Ignores links with a `ignore` or `data-ignore` attribute
    if (_browserEssentials.dom.hasAttr(target, 'ignore') || target.hasAttribute('target') || _browserEssentials.dom.hasAttr(target, 'href') === false) {
      return;
    }

    // Only continues if there is an `href` or `data-href` attribute present
    e.preventDefault();

    var href = _browserEssentials.dom.getAttr(target, 'href') || _browserEssentials.dom.getAttr(target, 'url');
    var location = _browserEssentials.url.resolve(this.state.location.href, href);

    // Tells the history handler to go to the new location
    this.props.history.goto(location);
  };

  Router.prototype.getContextProps = function getContextProps() {
    var contextProps = Object.assign({}, this.state, Router.defaultProps);

    (0, _optSetter.setDefinedOpt)(contextProps, this.props, false);
    contextProps.router = this;
    delete contextProps.maxClickSearchDepth;

    return contextProps;
  };

  Router.prototype.getRoute = function getRoute() {
    var routes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.state.routes;
    var location = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.state.location;

    for (var _iterator = routes, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref2;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref2 = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref2 = _i.value;
      }

      var _ref = _ref2;
      var path = _ref[0];
      var route = _ref[1];

      var match = routeMatcher.match(path, location.path);

      if (match !== void 0) {
        var _path = match[0],
            params = match[1];

        var view = route.view;
        /*
        console.log('Matched path:', path,
                    '\nMatched params:', params,
                    '\nMatched view:', view)
        */
        return { path: _path, params: params, view: view };
      }
    }

    throw new Error('Route not found for location: ' + ('' + this.state.location.href));
  };

  Router.prototype._getViewPropsFrom = function _getViewPropsFrom(props) {
    var viewProps = Object.assign({}, props);

    delete viewProps.routes;
    delete viewProps.location;

    return viewProps;
  };

  Router.prototype.render = function render() {
    var App = this.props.app;
    var props = this.getContextProps();
    var View = null;
    var viewProps = this._getViewPropsFrom(props);

    if (this.state.route.view) {
      View = this.state.route.view;
    }

    return _react2.default.createElement(
      _RouterContext2.default,
      props,
      _react2.default.createElement(
        App,
        viewProps,
        _react2.default.createElement(View, viewProps)
      )
    );
  };

  return Router;
}(_react2.default.Component), _class.displayName = 'Router', _class.defaultProps = {
  history: new _Html5History2.default(),
  initialLocation: null,
  maxClickSearchDepth: 10
}, _temp2);
exports.default = Router;