'use strict';

exports.__esModule = true;
exports.match = exports.findRoutes = undefined;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _routeParser = require('./routeParser');

var routeParser = _interopRequireWildcard(_routeParser);

var _getViewFromRoute = require('./getViewFromRoute');

var _getViewFromRoute2 = _interopRequireDefault(_getViewFromRoute);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var findRoutes = exports.findRoutes = function findRoutes(components) {
  var schemes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var routes = new Map();
  components = Array.isArray(components) ? components : [components];

  for (var _iterator = components, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var component = _ref;

    if (component.type === void 0) {
      continue;
    }

    var parentPaths = schemes.map(function (s) {
      return s.props.path || '';
    });
    var fullPath = _path2.default.join.apply(_path2.default, parentPaths.concat([component.props.path || '']));

    switch (component.type.displayName) {
      case 'Route':
      case 'Scheme':
        routes.set(fullPath, { name: component.props.name,
          schemes: [].concat(schemes),
          component: component,
          view: (0, _getViewFromRoute2.default)(component, schemes) });
        schemes.push(component);

        if (component.props.children) {
          var children = Array.isArray(component.props.children) ? component.props.children : [component.props.children];

          for (var _iterator2 = findRoutes(children, schemes), _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
            var _ref3;

            if (_isArray2) {
              if (_i2 >= _iterator2.length) break;
              _ref3 = _iterator2[_i2++];
            } else {
              _i2 = _iterator2.next();
              if (_i2.done) break;
              _ref3 = _i2.value;
            }

            var _ref2 = _ref3;
            var routePath = _ref2[0];
            var route = _ref2[1];

            routes.set(routePath, route);
          }
        }

        schemes.pop();
    }
  }
  return routes;
};

/** Determines if a route matches a given path
  * @param {String} path
  * @returns {Array} [route text, matched args, matched named args]
  */
var match = exports.match = function match(route, urlPath) {
  var pattern = routeParser.getPattern(route);
  var match = pattern.match.exec(urlPath);

  if (match) {
    var params = {};

    for (var i = 1; i < match.length; i++) {
      var g = decodeURIComponent(match[i]);
      var name = pattern.groups[i - 1][1];

      if (name !== undefined && name !== null && name.length) {
        params[name] = g;
      }
    }

    return [route, params];
  }
};