var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import path from 'path';
import * as routeParser from './routeParser';
import getViewFromRoute from './getViewFromRoute';

export var findRoutes = function findRoutes(components) {
  var schemes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var routes = new Map();
  components = Array.isArray(components) ? components : [components];

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = components[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var component = _step.value;

      if (component.type === void 0) {
        continue;
      }

      var parentPaths = schemes.map(function (s) {
        return s.props.path || '';
      });
      var fullPath = path.join.apply(path, _toConsumableArray(parentPaths).concat([component.props.path || '']));

      switch (component.type.displayName) {
        case 'Route':
        case 'Scheme':
          routes.set(fullPath, { name: component.props.name,
            schemes: [].concat(_toConsumableArray(schemes)),
            component: component,
            view: getViewFromRoute(component, schemes) });
          schemes.push(component);

          if (component.props.children) {
            var children = Array.isArray(component.props.children) ? component.props.children : [component.props.children];

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
              for (var _iterator2 = findRoutes(children, schemes)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var _ref = _step2.value;

                var _ref2 = _slicedToArray(_ref, 2);

                var routePath = _ref2[0];
                var route = _ref2[1];

                routes.set(routePath, route);
              }
            } catch (err) {
              _didIteratorError2 = true;
              _iteratorError2 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }
              } finally {
                if (_didIteratorError2) {
                  throw _iteratorError2;
                }
              }
            }
          }

          schemes.pop();
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return routes;
};

/** Determines if a route matches a given path
  * @param {String} path
  * @returns {Array} [route text, matched args, matched named args]
  */
export var match = function match(route, urlPath) {
  var pattern = routeParser.getPattern(route);
  var match = pattern.match.exec(urlPath);

  if (match) {
    var params = {};

    for (var i = 1; i < match.length; i++) {
      var g = decodeURIComponent(match[i]);
      var name = pattern.groups[i - 1][1];

      if (name !== undefined && name !== null && name.length) {
        params[name] = g;
      }
    }

    return [route, params];
  }
};