var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import escapeRegExp from 'lodash/escapeRegExp';
import memoize from 'lodash/memoize';
import routePeg from './routePeg';

export var shortcuts = {
  slug: '([a-z0-9\-\_]+)', // slug-format_1
  politeSlug: '([a-zA-Z0-9\-\_]+)', // Polite-_Form2
  strictSlug: '([a-z\-]+)', // strict-format
  username: '(\w{1,25})', // Cool_Kat43
  number: '([0-9]+)', // Any integer
  uid: '([0-9]{16,})', // Big integers
  alpha: '([a-zA-Z]+)', // AdELkf
  alnum: '([a-zA-Z0-9]+)', // alDKElff3812
  hashtag: '((%23|#)([^\s]+))', // %23GoForIt
  mention: '((%40|@)([^\s]+))', // %40ItIsMe
  word: '([a-zA-Z0-9+_]+)', // Az_09
  uuid: '([0-9A-F]{8}-' + '[0-9A-F]{4}-' + '[0-9A-F]{4}-' + '[89AB][0-9A-F]{3}-' + '[0-9A-F]{12})',
  hex: '([0-9a-fA-F]+)' // 7f14be654cfc...
};
shortcuts.id = shortcuts.number;
shortcuts.num = shortcuts.number;
shortcuts.user = shortcuts['politeSlug'];
shortcuts.username = shortcuts['politeSlug'];

var _getRegex = function _getRegex(regex) {
  regex = regex ? regex : '([^/]+)';
  var shortcutName = regex.slice(1, -1);

  if (shortcuts.hasOwnProperty(shortcutName)) {
    regex = shortcuts[shortcutName];
  }

  return regex;
};

var _getParam = function _getParam(param) {
  var rules = [];

  if (param.name.name) {
    rules.push(['', param.name.name, param.name.regex]);
  } else {
    rules.push([param.name, null, _getRegex(param.name)]);
  }

  rules.push(['=', null, escapeRegExp('=')]);

  if (param.value.name) {
    rules.push(['', param.value.name, param.value.regex]);
  } else {
    rules.push([param.value, null, _getRegex(param.value)]);
  }

  return rules;
};

export var parse = memoize(function (path) {
  var peg = routePeg.parse(path);
  var rules = [];

  for (var i = 0; i < peg.path.length; i++) {
    var _path = peg.path[i];
    var literal = '';
    var fieldName = void 0,
        regex = void 0;

    if (_path.name !== void 0) {
      fieldName = _path.name;
      regex = _getRegex(_path.regex);
    } else {
      literal = _path;
      fieldName = null;
      regex = '(' + escapeRegExp(_path) + ')';
    }

    rules.push([literal, fieldName, regex]);
  }

  if (peg.query.length) {
    for (var x in peg.query) {
      var param = peg.query[x];
      var sym = x === '0' ? '\?' : '\&';
      rules.push.apply(rules, [[sym, null, sym]].concat(_toConsumableArray(_getParam(param))));
    }
  }

  return rules;
});

/** Gets the regex rules from route path strings
  * @param {String} path
  * @returns {Object} {match: ..., groups: ...}
  */
export var getPattern = memoize(function (path) {
  var rules = parse(path);
  var pattern = [];
  var groups = [];

  for (var i = 0; i < rules.length; i++) {
    var rule = rules[i];

    var _rule = _slicedToArray(rule, 3),
        lit = _rule[0],
        fieldName = _rule[1],
        regex = _rule[2];

    pattern.push(regex);
    groups.push([regex, fieldName]);
  }

  var result = {
    match: new RegExp('^' + pattern.join('') + '(?:\\?|(?:\\/(?:\\?|$))|$)'),
    groups: groups
  };

  return result;
});

var _validOptData = function _validOptData(paramRegex, data) {
  var escapedData = escapeRegExp(data);
  return !paramRegex || paramRegex && new RegExp(paramRegex, 'g').test(escapedData);
};

export var format = function format(path) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var assertValid = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  var url = '';
  var parsedPath = parse(path);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = parsedPath[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref = _step.value;

      var _ref2 = _slicedToArray(_ref, 3);

      var lit = _ref2[0];
      var paramName = _ref2[1];
      var paramRegex = _ref2[2];

      url += lit || '';
      if (paramName) {
        if (data[paramName] === null || data[paramName] === void 0) {
          throw new Error('Missing parameter \'' + paramName + '\' for ' + path);
        }

        var optVal = String(data[paramName]);
        if (assertValid || _validOptData(paramRegex, optVal)) {
          url += optVal;
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return url;
};