import {url} from 'browser-essentials'
import Emitter from 'emitter-extended'


export default class HashHistory extends Emitter {
  location = url.fromHash()

  constructor (history_) {
    super('HashHistory')
    this.register('navigate', 'routerUpdated')
    this._hashChangeListener = this.handleHashChange.bind(this)
    window.addEventListener('hashchange', this._hashChangeListener)
    this.on('navigate', this.handleNavigate.bind(this))
  }

  onRouterUpdate (fn) {
    return this.on('routerUpdated', fn)
  }

  handleHashChange (e) {
    if (e.newURL !== e.oldURL) {
      this.emit('navigate', url.fromHash())
    }
  }

  handleNavigate () {
    this.location = url.fromHash()
  }

  goto (url_, state = null, title = null) {
    window.location.hash =  `#${url.parse(url_).path}`
  }

  toObject () {
    const out = {}

    for (let key in context.historyContext) {
      out[key] = this[key]
    }

    return out
  }

  dispose () {
    this.removeAllListeners()
    window.removeEventListener('hashchange', this._hashChangeListener)
  }
}
