import {url} from 'browser-essentials'
import Emitter from 'emitter-extended'
import * as context from './context'


export default class Html5History extends Emitter {
  location = url.fromWindow()

  constructor (history_) {
    super('Html5History')
    this.history = history_ || window.history || history
    this.register('navigate', 'routerUpdated')
    this._popStateListener = this.handlePopState.bind(this)
    window.addEventListener('popstate', this._popStateListener)
    this.on('navigate', this.handleNavigate.bind(this))
  }

  onRouterUpdate (fn) {
    return this.on('routerUpdated', fn)
  }

  handlePopState () {
    this.emit('navigate', url.fromWindow())
  }

  handleNavigate () {
    this.location = url.fromWindow()
  }

  push (...args) {this.history.pushState(...args)}
  get () {return this.history.state}
  back () {this.history.go(-1)}
  forward () {this.history.go(1)}

  // This needs to be updated to match the API of goto or a new method
  // needs to be added
  replace (...args) {this.history.replaceState(...args)}

  goto (url_, state = null, title = null) {
    const parsed = url.parse(url_)

    if (parsed.href !== this.location.href) {
      this.push(state, title, url_)
    }

    this.emit('navigate', parsed)
  }

  toObject () {
    const out = {}

    for (let key in context.historyContext) {
      out[key] = this[key]
    }

    return out
  }

  dispose () {
    this.removeAllListeners()
    window.removeEventListener('popstate', this._popStateListener)
  }
}
