import routeIsNamed from './routeIsNamed'
import routeIsNamedLike from './routeIsNamedLike'


const _cache = new WeakMap()
const _getCache = routes => {
  if (_cache.has(routes)) {
    return _cache.get(routes)
  }

  const c = {}
  _cache.set(routes, c)

  return c
}


const _get = (routes, name) => {
  let outPath
  const matches = []

  for (let [path, route] of routes.entries()) {
    // Either the absolute route name is matched or the route.name is matched
    if (routeIsNamed(route, name)) {
      outPath = path
    }

    // Partial matches
    if (routeIsNamedLike(route, name)) {
      matches.push(path)
    }
  }

  if (outPath === void 0 && matches.length) {
    outPath = matches[0]
  }

  return outPath
}


export default (routes, name) => {
  const cache = _getCache(routes)

  const getter = () => {
    if (cache.hasOwnProperty(name)) {
      return cache[name]
    }
    
    cache[name] = _get(routes, name)
    return cache[name]
  }

  return getter()
}
